use std::ops::Deref;
use std::path::Path;
use std::{error::Error, fs};

use clap::{ArgGroup, Parser};
use common::{Day, Part};

#[derive(Parser, Debug)]
#[command(about)]
#[command(group(
ArgGroup::new("input").required(true).args(["input_file", "input_str"])))]
struct Args {
    #[arg(short, long)]
    day: Day,
    #[arg(short, long)]
    part: Part,

    #[arg(short = 'f', long)]
    input_file: Option<String>,

    input_str: Option<String>,
}

fn main() -> Result<(), Box<dyn Error>> {
    let cli = Args::parse();

    let input_str = if let Some(inp) = cli.input_file.as_deref() {
        fs::read_to_string(Path::new(inp))?
    } else {
        cli.input_str.unwrap()
    };

    match (cli.day, cli.part) {
        (Day::One, part) => {
            println!("{}", day_01::run_part(part, input_str).unwrap().deref());
            Ok(())
        }
        (Day::Two, part) => {
            println!("{}", day_02::run_part(part, input_str).unwrap().deref());
            Ok(())
        }

        #[allow(unreachable_patterns)]
        (_, _) => {
            unimplemented!()
        }
    }
}
